Package: Pedixplorer
Version: 1.8.0
Date: 2025-08-26
Title: Pedigree Functions
Authors@R: c(
    person("Louis", "Le Nezet", email="louislenezet@gmail.com",
        role=c("aut","cre","ctb"), comment = c(ORCID = "0009-0000-0202-2703")),
	person("Jason", "Sinnwell", email="sinnwell.jason@mayo.edu", role="aut"),
	person("Terry", "Therneau", role="aut"),
	person("Daniel", "Schaid", role="ctb"),
	person("Elizabeth", "Atkinson", role="ctb"))
Depends: R (>= 4.4.0)
Imports: graphics, stats, methods, ggplot2, utils, grDevices, stringr,
        plyr, dplyr, tidyr, quadprog, Matrix, S4Vectors, shiny, readxl,
        DT, igraph, shinycssloaders, shinyhelper, shinyjs, shinyjqui,
        shinyWidgets, htmlwidgets, plotly, colourpicker, shinytoastr
Description: Routines to handle family data with a Pedigree object. The initial
    purpose was to create correlation structures that describe family
    relationships such as kinship and identity-by-descent, which can be used to
    model family data in mixed effects models, such as in the coxme function.
    Also includes a tool for Pedigree drawing which is focused on producing
    compact layouts without intervention. Recent additions include utilities to
    trim the Pedigree object with various criteria, and kinship for the X
    chromosome.
License: Artistic-2.0
Encoding: UTF-8
RoxygenNote: 7.3.3
Roxygen: list(markdown = TRUE)
VignetteBuilder: knitr
Suggests: diffviewer, gridExtra, testthat (>= 3.0.0), vdiffr,
        rmarkdown, BiocStyle, knitr, withr, qpdf, shinytest2, devtools,
        R.devices, usethis, rlang, magick, cowplot
Config/testthat/edition: 3
biocViews: Software, DataRepresentation, Genetics, GraphAndNetwork,
        Visualization
BugReports: https://github.com/LouisLeNezet/Pedixplorer/issues
URL: https://louislenezet.github.io/Pedixplorer/
BiocType: Software
Collate: 'AllValidity.R' 'AllClass.R' 'kindepth.R' 'kinship.R'
        'utils.R' 'AllConstructor.R' 'AllAccessors.R' 'AllGeneric.R'
        'Pedixplorer-package.R' 'alignped4.R' 'alignped3.R'
        'alignped2.R' 'alignped1.R' 'auto_hint.R' 'align.R' 'app.R'
        'app_color_picker.R' 'app_data_col_sel.R' 'app_data_download.R'
        'app_data_import.R' 'app_utils.R' 'app_family_sel.R'
        'app_health_sel.R' 'app_inf_sel.R' 'app_ped_avaf_infos.R'
        'app_plot_resize.R' 'app_plot_download.R' 'app_plot_legend.R'
        'app_plot_ped.R' 'app_plot_all.R' 'app_server.R' 'app_ui.R'
        'best_hint.R' 'bit_size.R' 'data.R' 'descendants.R'
        'family_check.R' 'find_unavailable.R' 'find_avail_affected.R'
        'find_avail_noninform.R' 'fix_parents.R' 'generate_aff_inds.R'
        'generate_colors.R' 'ibd_matrix.R' 'is_informative.R'
        'make_famid.R' 'min_dist_inf.R' 'norm_data.R' 'num_child.R'
        'ped_to_legdf.R' 'ped_to_plotdf.R' 'plot_fct.R' 'plot_fromdf.R'
        'plot_pedigree.R' 'shrink.R' 'unrelated.R' 'useful_inds.R'
LazyData: false
git_url: https://git.bioconductor.org/packages/Pedixplorer
git_branch: RELEASE_3_23
git_last_commit: 98d7b52
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:06:36 UTC; biocbuild
Author: Louis Le Nezet [aut, cre, ctb] (ORCID:
    <https://orcid.org/0009-0000-0202-2703>),
  Jason Sinnwell [aut],
  Terry Therneau [aut],
  Daniel Schaid [ctb],
  Elizabeth Atkinson [ctb]
Maintainer: Louis Le Nezet <louislenezet@gmail.com>
Built: R 4.6.0; ; 2026-04-29 04:54:21 UTC; unix
