# Generated by roxygen2: do not edit by hand

export("adopted<-")
export("affected<-")
export("asymptomatic<-")
export("avail<-")
export("border<-")
export("carrier<-")
export("consultand<-")
export("dadid<-")
export("dateofbirth<-")
export("dateofdeath<-")
export("deceased<-")
export("evaluated<-")
export("famid<-")
export("fertility<-")
export("fill<-")
export("hints<-")
export("horder<-")
export("id<-")
export("isinf<-")
export("kin<-")
export("miscarriage<-")
export("momid<-")
export("ped<-")
export("proband<-")
export("rel<-")
export("scales<-")
export("sex<-")
export("spouse<-")
export("useful<-")
export(Hints)
export(Ped)
export(Pedigree)
export(Rel)
export(Scales)
export(adopted)
export(affected)
export(align)
export(ancestors)
export(asymptomatic)
export(auto_hint)
export(avail)
export(best_hint)
export(bit_size)
export(border)
export(carrier)
export(code)
export(color_picker_demo)
export(color_picker_server)
export(compute_stress)
export(consultand)
export(dadid)
export(data_col_sel_demo)
export(data_download_demo)
export(data_import_demo)
export(dateofbirth)
export(dateofdeath)
export(deceased)
export(descendants)
export(eval_perm)
export(evaluated)
export(famid)
export(family_check)
export(family_sel_demo)
export(fertility)
export(fertility_to_factor)
export(fill)
export(find_avail_affected)
export(find_avail_noninform)
export(find_unavailable)
export(fix_parents)
export(generate_aff_inds)
export(generate_border)
export(generate_colors)
export(generate_fill)
export(get_famid)
export(get_families_table)
export(get_title)
export(health_sel_demo)
export(hints)
export(horder)
export(ibd_matrix)
export(id)
export(id1)
export(id2)
export(inf_sel_demo)
export(is_disconnected)
export(is_founder)
export(is_informative)
export(isinf)
export(kin)
export(kindepth)
export(kinship)
export(make_famid)
export(make_reactive)
export(min_dist_inf)
export(miscarriage)
export(miscarriage_to_factor)
export(momid)
export(norm_ped)
export(norm_rel)
export(num_child)
export(parent_of)
export(ped)
export(ped_avaf_infos_demo)
export(ped_server)
export(ped_shiny)
export(ped_to_legdf)
export(ped_to_plotdf)
export(ped_ui)
export(plink_to_pedigree)
export(plot_download_demo)
export(plot_fromdf)
export(plot_legend_demo)
export(plot_legend_server)
export(plot_legend_ui)
export(plot_ped_demo)
export(plot_resize_server)
export(proband)
export(rel)
export(rel_code_to_factor)
export(rescale)
export(scales)
export(sex)
export(sex_to_factor)
export(shrink)
export(spouse)
export(unrelated)
export(upd_famid)
export(useful)
export(useful_inds)
export(vect_to_binary)
exportClasses(Hints)
exportClasses(Ped)
exportClasses(Pedigree)
exportClasses(Rel)
exportClasses(Scales)
exportMethods("[")
exportMethods("adopted<-")
exportMethods("affected<-")
exportMethods("asymptomatic<-")
exportMethods("avail<-")
exportMethods("border<-")
exportMethods("carrier<-")
exportMethods("consultand<-")
exportMethods("dadid<-")
exportMethods("dateofbirth<-")
exportMethods("dateofdeath<-")
exportMethods("deceased<-")
exportMethods("evaluated<-")
exportMethods("famid<-")
exportMethods("fertility<-")
exportMethods("fill<-")
exportMethods("hints<-")
exportMethods("horder<-")
exportMethods("id<-")
exportMethods("isinf<-")
exportMethods("kin<-")
exportMethods("mcols<-")
exportMethods("miscarriage<-")
exportMethods("momid<-")
exportMethods("ped<-")
exportMethods("proband<-")
exportMethods("rel<-")
exportMethods("scales<-")
exportMethods("sex<-")
exportMethods("spouse<-")
exportMethods("useful<-")
exportMethods(Hints)
exportMethods(Ped)
exportMethods(Pedigree)
exportMethods(Rel)
exportMethods(Scales)
exportMethods(adopted)
exportMethods(affected)
exportMethods(as.data.frame)
exportMethods(as.list)
exportMethods(asymptomatic)
exportMethods(auto_hint)
exportMethods(avail)
exportMethods(border)
exportMethods(carrier)
exportMethods(code)
exportMethods(consultand)
exportMethods(dadid)
exportMethods(dateofbirth)
exportMethods(dateofdeath)
exportMethods(deceased)
exportMethods(evaluated)
exportMethods(famid)
exportMethods(family_check)
exportMethods(fertility)
exportMethods(fill)
exportMethods(find_avail_affected)
exportMethods(find_avail_noninform)
exportMethods(find_unavailable)
exportMethods(fix_parents)
exportMethods(generate_colors)
exportMethods(hints)
exportMethods(horder)
exportMethods(id)
exportMethods(id1)
exportMethods(id2)
exportMethods(is_informative)
exportMethods(is_parent)
exportMethods(isinf)
exportMethods(kin)
exportMethods(kinship)
exportMethods(length)
exportMethods(mcols)
exportMethods(min_dist_inf)
exportMethods(miscarriage)
exportMethods(momid)
exportMethods(num_child)
exportMethods(parent_of)
exportMethods(ped)
exportMethods(ped_to_legdf)
exportMethods(ped_to_plotdf)
exportMethods(plot)
exportMethods(proband)
exportMethods(rel)
exportMethods(scales)
exportMethods(sex)
exportMethods(show)
exportMethods(shrink)
exportMethods(spouse)
exportMethods(subset)
exportMethods(summary)
exportMethods(unrelated)
exportMethods(upd_famid)
exportMethods(useful)
exportMethods(useful_inds)
importFrom(DT,dataTableOutput)
importFrom(DT,datatable)
importFrom(DT,renderDataTable)
importFrom(Matrix,bdiag)
importFrom(Matrix,forceSymmetric)
importFrom(Matrix,sparseMatrix)
importFrom(S4Vectors,'mcols')
importFrom(S4Vectors,'mcols<-')
importFrom(S4Vectors,as.data.frame)
importFrom(S4Vectors,as.list)
importFrom(S4Vectors,cbind_mcols_for_display)
importFrom(S4Vectors,classNameForDisplay)
importFrom(S4Vectors,get_showHeadLines)
importFrom(S4Vectors,makeClassinfoRowForCompactPrinting)
importFrom(S4Vectors,makeNakedCharacterMatrixForDisplay)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,parallel_slot_names)
importFrom(S4Vectors,subset)
importFrom(S4Vectors,summary)
importFrom(colourpicker,colourInput)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,everything)
importFrom(dplyr,group_by)
importFrom(dplyr,join_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_at)
importFrom(dplyr,mutate_if)
importFrom(dplyr,n)
importFrom(dplyr,one_of)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,sym)
importFrom(dplyr,ungroup)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,scale_y_reverse)
importFrom(ggplot2,theme)
importFrom(ggplot2,unit)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(graphics,frame)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,polygon)
importFrom(graphics,segments)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(htmlwidgets,saveWidget)
importFrom(igraph,components)
importFrom(igraph,graph_from_data_frame)
importFrom(methods,'slot<-')
importFrom(methods,as)
importFrom(methods,callNextMethod)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,show)
importFrom(methods,slotNames)
importFrom(methods,validObject)
importFrom(plotly,config)
importFrom(plotly,ggplotly)
importFrom(plotly,layout)
importFrom(plotly,plotlyOutput)
importFrom(plotly,renderPlotly)
importFrom(plyr,rbind.fill)
importFrom(plyr,revalue)
importFrom(quadprog,solve.QP)
importFrom(readxl,excel_sheets)
importFrom(readxl,read_excel)
importFrom(shiny,HTML)
importFrom(shiny,NS)
importFrom(shiny,actionButton)
importFrom(shiny,bindEvent)
importFrom(shiny,checkboxInput)
importFrom(shiny,column)
importFrom(shiny,debounce)
importFrom(shiny,div)
importFrom(shiny,downloadButton)
importFrom(shiny,downloadHandler)
importFrom(shiny,eventReactive)
importFrom(shiny,exportTestValues)
importFrom(shiny,fileInput)
importFrom(shiny,fluidPage)
importFrom(shiny,fluidRow)
importFrom(shiny,h2)
importFrom(shiny,h3)
importFrom(shiny,h5)
importFrom(shiny,hr)
importFrom(shiny,htmlOutput)
importFrom(shiny,icon)
importFrom(shiny,is.reactive)
importFrom(shiny,isolate)
importFrom(shiny,modalDialog)
importFrom(shiny,moduleServer)
importFrom(shiny,numericInput)
importFrom(shiny,observe)
importFrom(shiny,observeEvent)
importFrom(shiny,plotOutput)
importFrom(shiny,radioButtons)
importFrom(shiny,reactive)
importFrom(shiny,reactiveValues)
importFrom(shiny,removeModal)
importFrom(shiny,renderPlot)
importFrom(shiny,renderPrint)
importFrom(shiny,renderTable)
importFrom(shiny,renderText)
importFrom(shiny,renderUI)
importFrom(shiny,req)
importFrom(shiny,selectInput)
importFrom(shiny,shinyApp)
importFrom(shiny,shinyServer)
importFrom(shiny,shinyUI)
importFrom(shiny,showModal)
importFrom(shiny,sliderInput)
importFrom(shiny,stopApp)
importFrom(shiny,strong)
importFrom(shiny,tableOutput)
importFrom(shiny,tagList)
importFrom(shiny,tags)
importFrom(shiny,textAreaInput)
importFrom(shiny,textOutput)
importFrom(shiny,titlePanel)
importFrom(shiny,uiOutput)
importFrom(shinyWidgets,switchInput)
importFrom(shinyWidgets,updateSwitchInput)
importFrom(shinycssloaders,withSpinner)
importFrom(shinyhelper,helper)
importFrom(shinyhelper,observe_helpers)
importFrom(shinyjqui,jqui_resizable)
importFrom(shinyjs,useShinyjs)
importFrom(shinytoastr,toastr_error)
importFrom(shinytoastr,toastr_info)
importFrom(shinytoastr,toastr_success)
importFrom(shinytoastr,toastr_warning)
importFrom(shinytoastr,useToastr)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stringr,str_detect)
importFrom(stringr,str_remove_all)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split_fixed)
importFrom(stringr,str_split_i)
importFrom(stringr,str_to_title)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,spread)
importFrom(tidyr,unite)
importFrom(tools,file_ext)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(utils,write.csv2)
