#### Select the **availability status** column of the individuals

**Optional** column.

The value of this column must contains a logical vector with the availability status of the individuals:

- `FALSE` = not available
- `TRUE` = available
- `NA` = unknown
