#### Select the **evaluated status** column of the individuals

**Optional** column.

The value of this column must contains a logical vector with the evaluation status of the individuals:

- `FALSE` = documented evaluation not available,
- `TRUE` = documented evaluation available
