#### Select the **proband status** column of the individuals

**Optional** column.

The value of this column must contains a logical vector with the proband status of the individuals.
A proband being an affected family member coming to medical attention independent of other family members.

- `FALSE` = not a proband
- `TRUE` = proband
