### Select The maximum stress level to accept

This value control how much stress is tolerated in the generated pedigree.
A lower value means a more well designed pedigree but might take times to obtain or might even never be attained.
