#### Select the **adoption status** column of the individuals

**Optional** column.

The value of this column must contains a logical vector with the adopted status of the individuals.

- `FALSE` = not adopted
- `TRUE` = adopted
- `NA` = unknown
