#### Select the **family identifier** column of the individuals

**Optional** column.

The value of this column must contains a character vector with no underscore `_` with the family identifiers of the individuals.

If provide, will be aggregated to the individuals identifiers separated by an underscore.
