### How to Use This App

#### 1. Upload your pedigree file and select the corresponding columns names

- The app supports various file formats including `.csv`, `.tsv`, `.fam` or `.xlsx`.
- A test dataset is provided for you to explore the app's features.
- Ensure that at least the columns for `id`, `dadid`, `momid` and `sex` are present.

#### 2. Select the health options

- This will be used to categorise the individuals between affected and unaffected as
well as how their colors will be displayed in the pedigree.

#### 3. Select the main family you want to plot

- The app will provide a list of all the families in the dataset and their respective number of individuals as well as the major mode for the column selected.
- The Health & Availability table on the right is then updated to show the selected family.

#### 4. (Optional) If the family is large, you can select a subset of individuals to plot

- This is useful for families with many individuals or when you want to focus on a specific subset of the family.
- Select which individuals are informative and how close the other individuals needs to be to them to be included.

#### 5. (Optional) Set the plot to be **interactive**
- This allows you to hover over individuals to see their details and navigate through the pedigree more easily.
- You can add additional information to the plot by selecting the columns you want to display.

<div align="center" style="font-size:24px; font-style: italic;">
Happy exploring!
</div>