#### Select the **second individuals' identifier** column

**Mandatory** column.

The value of this column must contains a character vector with no underscore `_` with the id of the second individuals.
