#### Select the **deceased status** column of the individuals

**Optional** column.

The value of this column must contains a logical vector with the death status of the individuals:

- `FALSE` = alive
- `TRUE` = dead
- `NA` = unknown
