#### Select the **consultand status** column of the individuals

**Optional** column.

The value of this column must contains a logical vector with the consultand status of the individuals.
A consultand being an individual seeking genetic counseling/testing.

- `FALSE` = not a consultand
- `TRUE` = consultand
