#### Select the **individuals' identifier** column

**Mandatory** column.

The value of this column must contains a character vector with no underscore `_` with the id of each individuals.
The value of this column must be unique for each individual.
