#### Select the **asymptomatic status** column of the individuals

**Optional** column.

The value of this column must contains a logical vector with the asymptomatic status of the individuals.
An asymptomatic individual being an individual clinically unaffected at this time but could later exhibit symptoms.

- `FALSE` = not asymptomatic
- `TRUE` = asymptomatic
- `NA` = unknown
