#### Select the **carrier status** column of the individuals

**Optional** column.

The value of this column must contains a logical vector with the carrier status of the individuals.
A carrier being an individual who has the genetic trait but who is not likely to manifest the disease regardless of inheritance pattern

- `FALSE` = not carrier
- `TRUE` = carrier
- `NA` = unknown
