### Select which individuals are informative

This filter allows you to set which set of individuals are informative.
You can either select a set of individuals :

- `All individuals`: All individuals in the dataset are informative this will plot all the individuals in the dataset.
- `Available or Affected`: Only the individuals that are available or affected are informative.
- `Available only`: Only the individuals that are available are informative.
- `Affected only`: Only the individuals that are affected are informative.
- `Available and Affected`: Only the individuals that are available and affected are informative.

You can also select a custom set of individuals by providing a vector of values that will be compared to the selected column.

- `Custom`: Only the individuals with the value selected in the custom column 