#### Select the **sex** column of the individuals

**Mandatory** column.

The value of this column must contains a character, factor or numeric vector corresponding to the gender of the individuals.

This will be transformed to an ordered factor with the following levels:
`male` < `female` < `unknown`

The following values are recognized:
- "male": "m", "male", "man", `1`
- "female": "f", "female", "woman", `2`
- "unknown": "unknown", `3`
