### Select which family to continue with

The table provides a list of all the families in the dataset and their respective number of individuals as well as the major mode for the column selected.

The Health & Availability table on the right is then updated to show the selected family.
