#### Select the **mother identifier** column of the individuals

**Mandatory** column.

The value of this column must contains a character vector with no underscore `_` with the id of the mother of the individuals.
If the individual has no mother, the value of this column must be empty or `NA` or `0`
