### Select the pedigree data file

The file **must** contains the following columns :

- `id` : individual identifier
- `dadid` : father identifier
- `momid` : mother identifier
- `sex` : individual sex

Other columns are **optional** and can be used to provide additional information about the individuals:

- `famid` : family identifier
- `fertility` : fertility status
- `miscarriage` : miscarriage status
- `deceased` : deceased status
- `avail` : availability status
- `evaluated` : evaluated status
- `consultand` : consultand status
- `proband` : proband status
- `carrier` : carrier status
- `asymptomatic` : asymptomatic status
- `adopted` : adopted status
- `dateofbirth` : date of birth
- `dateofdeath` : date of death
