# Pedigree legend works

    Code
      lst
    Output
      $df
                          id    x0     y0 x1 y1        type    fill border angle
      1               titles 0.000 0.0000 NA NA        text   black   <NA>    NA
      2               titles 2.057 0.0000 NA NA        text   black   <NA>    NA
      3               titles 4.145 0.0000 NA NA        text   black   <NA>    NA
      4               titles 6.268 0.0000 NA NA        text   black   <NA>    NA
      5               titles 8.389 0.0000 NA NA        text   black   <NA>    NA
      6                  sex 0.000 0.8333 NA NA  square_1_1   white  black    NA
      7                  sex 0.000 2.1670 NA NA  circle_1_1   white  black    NA
      8                  sex 0.000 3.5000 NA NA diamond_1_1   white  black    NA
      9            sex_label 1.000 1.3330 NA NA        text   black   <NA>    NA
      10           sex_label 1.000 2.6670 NA NA        text   black   <NA>    NA
      11           sex_label 1.000 4.0000 NA NA        text   black   <NA>    NA
      12              border 2.057 0.8333 NA NA  square_1_1   white  black    NA
      13              border 2.057 2.1670 NA NA  square_1_1   white  green    NA
      14        border_label 3.057 1.3330 NA NA        text   black   <NA>    NA
      15        border_label 3.057 2.6670 NA NA        text   black   <NA>    NA
      16         aff_bkg_1_0 4.145 0.8333 NA NA  square_1_1   white  black    NA
      17         aff_bkg_1_1 4.145 2.1670 NA NA  square_1_1   white  black    NA
      18        aff_bkg_1_NA 4.145 3.5000 NA NA  square_1_1   white  black    NA
      19        affected_1_0 4.145 0.8333 NA NA  square_3_1   white  black    NA
      20        affected_1_1 4.145 2.1670 NA NA  square_3_1     red  black    NA
      21       affected_1_NA 4.145 3.5000 NA NA  square_3_1    grey  black    NA
      22  affected_label_1_0 5.145 1.3330 NA NA        text   black   <NA>    NA
      23  affected_label_1_1 5.145 2.6670 NA NA        text   black   <NA>    NA
      24 affected_label_1_NA 5.145 4.0000 NA NA        text   black   <NA>    NA
      25         aff_bkg_2_0 6.268 0.8333 NA NA  square_1_1   white  black    NA
      26         aff_bkg_2_1 6.268 2.1670 NA NA  square_1_1   white  black    NA
      27        affected_2_0 6.268 0.8333 NA NA  square_3_2   white  black    NA
      28        affected_2_1 6.268 2.1670 NA NA  square_3_2     red  black    NA
      29  affected_label_2_0 7.268 1.3330 NA NA        text   black   <NA>    NA
      30  affected_label_2_1 7.268 2.6670 NA NA        text   black   <NA>    NA
      31         aff_bkg_3_1 8.389 0.8333 NA NA  square_1_1   white  black    NA
      32         aff_bkg_3_2 8.389 2.1670 NA NA  square_1_1   white  black    NA
      33         aff_bkg_3_3 8.389 3.5000 NA NA  square_1_1   white  black    NA
      34         aff_bkg_3_4 8.389 4.8330 NA NA  square_1_1   white  black    NA
      35         aff_bkg_3_5 8.389 6.1670 NA NA  square_1_1   white  black    NA
      36         aff_bkg_3_6 8.389 7.5000 NA NA  square_1_1   white  black    NA
      37        affected_3_1 8.389 0.8333 NA NA  square_3_3 #FFFFFF  black    NA
      38        affected_3_2 8.389 2.1670 NA NA  square_3_3 #9AB1C4  black    NA
      39        affected_3_3 8.389 3.5000 NA NA  square_3_3 #36648B  black    NA
      40        affected_3_4 8.389 4.8330 NA NA  square_3_3 #FFC0CB  black    NA
      41        affected_3_5 8.389 6.1670 NA NA  square_3_3 #CF70DD  black    NA
      42        affected_3_6 8.389 7.5000 NA NA  square_3_3 #A020F0  black    NA
      43  affected_label_3_1 9.389 1.3330 NA NA        text   black   <NA>    NA
      44  affected_label_3_2 9.389 2.6670 NA NA        text   black   <NA>    NA
      45  affected_label_3_3 9.389 4.0000 NA NA        text   black   <NA>    NA
      46  affected_label_3_4 9.389 5.3330 NA NA        text   black   <NA>    NA
      47  affected_label_3_5 9.389 6.6670 NA NA        text   black   <NA>    NA
      48  affected_label_3_6 9.389 8.0000 NA NA        text   black   <NA>    NA
      49             max_lim 0.000 0.0000 NA NA        text    <NA>  black    NA
      50             max_lim 8.389 8.0000 NA NA        text    <NA>  black    NA
         density cex                         label tips adjx adjy lwd
      1       NA 1.2                           Sex <NA>    0    1  NA
      2       NA 1.2                        Border <NA>    0    1  NA
      3       NA 1.2                     affection <NA>    0    1  NA
      4       NA 1.2                         avail <NA>    0    1  NA
      5       NA 1.2                       val_num <NA>    0    1  NA
      6       NA  NA                          <NA> <NA>   NA   NA   1
      7       NA  NA                          <NA> <NA>   NA   NA   1
      8       NA  NA                          <NA> <NA>   NA   NA   1
      9       NA 0.8                          Male <NA>    0    1  NA
      10      NA 0.8                        Female <NA>    0    1  NA
      11      NA 0.8                       Unknown <NA>    0    1  NA
      12      NA  NA                          <NA> <NA>   NA   NA   1
      13      NA  NA                          <NA> <NA>   NA   NA   1
      14      NA 0.8                 Non Available <NA>    0    1  NA
      15      NA 0.8                     Available <NA>    0    1  NA
      16      NA  NA                          <NA> <NA>   NA   NA   1
      17      NA  NA                          <NA> <NA>   NA   NA   1
      18      NA  NA                          <NA> <NA>   NA   NA   1
      19      NA  NA                          <NA> <NA>   NA   NA   1
      20      NA  NA                          <NA> <NA>   NA   NA   1
      21      NA  NA                          <NA> <NA>   NA   NA   1
      22      NA 0.8             Healthy <= to 0.5 <NA>    0    1  NA
      23      NA 0.8             Affected > to 0.5 <NA>    0    1  NA
      24      NA 0.8                            NA <NA>    0    1  NA
      25      NA  NA                          <NA> <NA>   NA   NA   1
      26      NA  NA                          <NA> <NA>   NA   NA   1
      27      NA  NA                          <NA> <NA>   NA   NA   1
      28      NA  NA                          <NA> <NA>   NA   NA   1
      29      NA 0.8             Healthy are FALSE <NA>    0    1  NA
      30      NA 0.8             Affected are TRUE <NA>    0    1  NA
      31      NA  NA                          <NA> <NA>   NA   NA   1
      32      NA  NA                          <NA> <NA>   NA   NA   1
      33      NA  NA                          <NA> <NA>   NA   NA   1
      34      NA  NA                          <NA> <NA>   NA   NA   1
      35      NA  NA                          <NA> <NA>   NA   NA   1
      36      NA  NA                          <NA> <NA>   NA   NA   1
      37      NA  NA                          <NA> <NA>   NA   NA   1
      38      NA  NA                          <NA> <NA>   NA   NA   1
      39      NA  NA                          <NA> <NA>   NA   NA   1
      40      NA  NA                          <NA> <NA>   NA   NA   1
      41      NA  NA                          <NA> <NA>   NA   NA   1
      42      NA  NA                          <NA> <NA>   NA   NA   1
      43      NA 0.8 Healthy <= to 115 : [101,106] <NA>    0    1  NA
      44      NA 0.8 Healthy <= to 115 : (106,110] <NA>    0    1  NA
      45      NA 0.8 Healthy <= to 115 : (110,115] <NA>    0    1  NA
      46      NA 0.8 Affected > to 115 : [116,124] <NA>    0    1  NA
      47      NA 0.8 Affected > to 115 : (124,133] <NA>    0    1  NA
      48      NA 0.8 Affected > to 115 : (133,141] <NA>    0    1  NA
      49      NA  NA                          <NA> <NA>    0    1  NA
      50      NA  NA                          <NA> <NA>    0    1  NA
      
      $par_usr
      $par_usr$boxh
      [1] 1
      
      $par_usr$boxw
      [1] 1
      
      $par_usr$cex
      [1] 0.8
      
      $par_usr$usr
      [1] 0.000 9.389 0.000 8.000
      
      

