# Pedigree plotting test

    Code
      lst
    Output
      $df
                               id       x0    y0      x1     y1       type    fill
      1                   polygon  1.00000 1.000      NA     NA square_2_1     red
      2                   polygon  0.00000 2.000      NA     NA square_2_1   white
      3                   polygon  0.70710 3.000      NA     NA circle_2_1   white
      4                   polygon  1.20700 4.000      NA     NA circle_2_1   white
      5                   polygon  2.00000 1.000      NA     NA circle_2_1   white
      6                   polygon  1.00000 2.000      NA     NA circle_2_1     red
      7                   polygon  1.70700 3.000      NA     NA square_2_1     red
      8                   polygon  2.00000 2.000      NA     NA square_2_1   white
      9                   polygon  2.70700 3.000      NA     NA square_2_1   white
      10                  polygon  3.00000 2.000      NA     NA circle_2_1     red
      11                 aff_mark  0.96820 1.066      NA     NA       text   black
      12                 aff_mark -0.03177 2.066      NA     NA       text   black
      13                 aff_mark  0.67530 3.066      NA     NA       text   black
      14                 aff_mark  1.17500 4.066      NA     NA       text   black
      15                 aff_mark  1.96800 1.066      NA     NA       text   black
      16                 aff_mark  0.96820 2.066      NA     NA       text   black
      17                 aff_mark  1.67500 3.066      NA     NA       text   black
      18                 aff_mark  1.96800 2.066      NA     NA       text   black
      19                 aff_mark  2.67500 3.066      NA     NA       text   black
      20                 aff_mark  2.96800 2.066      NA     NA       text   black
      21                  polygon  1.00000 1.000      NA     NA square_2_2   white
      22                  polygon  0.00000 2.000      NA     NA square_2_2   white
      23                  polygon  0.70710 3.000      NA     NA circle_2_2 #c300ff
      24                  polygon  1.20700 4.000      NA     NA circle_2_2   white
      25                  polygon  2.00000 1.000      NA     NA circle_2_2    grey
      26                  polygon  1.00000 2.000      NA     NA circle_2_2 #c300ff
      27                  polygon  1.70700 3.000      NA     NA square_2_2   white
      28                  polygon  2.00000 2.000      NA     NA square_2_2 #c300ff
      29                  polygon  2.70700 3.000      NA     NA square_2_2   white
      30                  polygon  3.00000 2.000      NA     NA circle_2_2   white
      31                 aff_mark  1.03200 1.066      NA     NA       text   black
      32                 aff_mark  0.03177 2.066      NA     NA       text   black
      33                 aff_mark  0.73890 3.066      NA     NA       text   black
      34                 aff_mark  1.23900 4.066      NA     NA       text   black
      35                 aff_mark  2.03200 1.066      NA     NA       text   black
      36                 aff_mark  1.03200 2.066      NA     NA       text   black
      37                 aff_mark  1.73900 3.066      NA     NA       text   black
      38                 aff_mark  2.03200 2.066      NA     NA       text   black
      39                 aff_mark  2.73900 3.066      NA     NA       text   black
      40                 aff_mark  3.03200 2.066      NA     NA       text   black
      41                     dead  0.92370 1.145 1.07600 0.9868   segments   black
      42                     dead -0.07626 2.145 0.07626 1.9870   segments   black
      43                     dead  1.92400 1.145 2.07600 0.9868   segments   black
      44                     dead  0.92370 2.145 1.07600 1.9870   segments   black
      45                       id  1.00000 1.243      NA     NA       text   black
      46                       id  0.00000 2.243      NA     NA       text   black
      47                       id  0.70710 3.243      NA     NA       text   black
      48                       id  1.20700 4.243      NA     NA       text   black
      49                       id  2.00000 1.243      NA     NA       text   black
      50                       id  1.00000 2.243      NA     NA       text   black
      51                       id  1.70700 3.243      NA     NA       text   black
      52                       id  2.00000 2.243      NA     NA       text   black
      53                       id  2.70700 3.243      NA     NA       text   black
      54                       id  3.00000 2.243      NA     NA       text   black
      55             line_spouses  1.06400 1.066 1.93600 1.0660   segments   black
      56             line_spouses  0.06355 2.066 0.93640 2.0660   segments   black
      57             line_spouses  0.77060 3.066 1.64300 3.0660   segments   black
      58             line_spouses  2.06400 2.066 2.93600 2.0660   segments   black
      59            line_spouses2  0.77060 3.079 1.64300 3.0790   segments   black
      60   line_children_vertical  0.00000 2.000 0.00000 1.8020   segments   black
      61   line_children_vertical  3.00000 2.000 3.00000 1.8020   segments   black
      62 line_children_horizontal  0.00000 1.802 3.00000 1.8020   segments   black
      63          line_parent_mid  1.50000 1.802 1.50000 1.5810   segments   black
      64          line_parent_mid  1.50000 1.581 1.50000 1.2870   segments   black
      65          line_parent_mid  1.50000 1.287 1.50000 1.0660   segments   black
      66   line_children_vertical  0.70710 3.000 0.70710 2.8020   segments   black
      67 line_children_horizontal  0.70710 2.802 0.70710 2.8020   segments   black
      68          line_parent_mid  0.70710 2.802 0.70710 2.5810   segments   black
      69          line_parent_mid  0.70710 2.581 0.50000 2.2870   segments   black
      70          line_parent_mid  0.50000 2.287 0.50000 2.0660   segments   black
      71   line_children_vertical  1.70700 3.000 2.20700 2.8020   segments   black
      72   line_children_vertical  2.70700 3.000 2.20700 2.8020   segments   black
      73     label_children_twin3  2.20700 2.901      NA     NA       text   black
      74 line_children_horizontal  2.20700 2.802 2.20700 2.8020   segments   black
      75          line_parent_mid  2.20700 2.802 2.20700 2.5810   segments   black
      76          line_parent_mid  2.20700 2.581 2.50000 2.2870   segments   black
      77          line_parent_mid  2.50000 2.287 2.50000 2.0660   segments   black
      78   line_children_vertical  1.20700 4.000 1.20700 3.8020   segments   black
      79 line_children_horizontal  1.20700 3.802 1.20700 3.8020   segments   black
      80          line_parent_mid  1.20700 3.802 1.20700 3.5810   segments   black
      81          line_parent_mid  1.20700 3.581 1.20700 3.2870   segments   black
      82          line_parent_mid  1.20700 3.287 1.20700 3.0660   segments   black
         border angle density cex label
      1   black    NA      NA  NA  <NA>
      2   green    NA      NA  NA  <NA>
      3   black    NA      NA  NA  <NA>
      4   black    NA      NA  NA  <NA>
      5   green    NA      NA  NA  <NA>
      6   green    NA      NA  NA  <NA>
      7   green    NA      NA  NA  <NA>
      8   green    NA      NA  NA  <NA>
      9   green    NA      NA  NA  <NA>
      10  black    NA      NA  NA  <NA>
      11   <NA>    NA      NA   1     1
      12   <NA>    NA      NA   1     0
      13   <NA>    NA      NA   1     0
      14   <NA>    NA      NA   1     0
      15   <NA>    NA      NA   1     0
      16   <NA>    NA      NA   1     1
      17   <NA>    NA      NA   1     1
      18   <NA>    NA      NA   1     0
      19   <NA>    NA      NA   1     0
      20   <NA>    NA      NA   1     1
      21  black    NA      NA  NA  <NA>
      22  green    NA      NA  NA  <NA>
      23  black    NA      NA  NA  <NA>
      24  black    NA      NA  NA  <NA>
      25  green    NA      NA  NA  <NA>
      26  green    NA      NA  NA  <NA>
      27  green    NA      NA  NA  <NA>
      28  green    NA      NA  NA  <NA>
      29  green    NA      NA  NA  <NA>
      30  black    NA      NA  NA  <NA>
      31   <NA>    NA      NA   1     0
      32   <NA>    NA      NA   1     0
      33   <NA>    NA      NA   1     1
      34   <NA>    NA      NA   1     0
      35   <NA>    NA      NA   1  <NA>
      36   <NA>    NA      NA   1     1
      37   <NA>    NA      NA   1     0
      38   <NA>    NA      NA   1     1
      39   <NA>    NA      NA   1     0
      40   <NA>    NA      NA   1     0
      41   <NA>    NA      NA  NA  <NA>
      42   <NA>    NA      NA  NA  <NA>
      43   <NA>    NA      NA  NA  <NA>
      44   <NA>    NA      NA  NA  <NA>
      45   <NA>    NA      NA   1   1_1
      46   <NA>    NA      NA   1   1_3
      47   <NA>    NA      NA   1   1_7
      48   <NA>    NA      NA   1  1_10
      49   <NA>    NA      NA   1   1_2
      50   <NA>    NA      NA   1   1_5
      51   <NA>    NA      NA   1   1_8
      52   <NA>    NA      NA   1   1_6
      53   <NA>    NA      NA   1   1_9
      54   <NA>    NA      NA   1   1_4
      55   <NA>    NA      NA  NA  <NA>
      56   <NA>    NA      NA  NA  <NA>
      57   <NA>    NA      NA  NA  <NA>
      58   <NA>    NA      NA  NA  <NA>
      59   <NA>    NA      NA  NA  <NA>
      60   <NA>    NA      NA  NA  <NA>
      61   <NA>    NA      NA  NA  <NA>
      62   <NA>    NA      NA  NA  <NA>
      63   <NA>    NA      NA  NA  <NA>
      64   <NA>    NA      NA  NA  <NA>
      65   <NA>    NA      NA  NA  <NA>
      66   <NA>    NA      NA  NA  <NA>
      67   <NA>    NA      NA  NA  <NA>
      68   <NA>    NA      NA  NA  <NA>
      69   <NA>    NA      NA  NA  <NA>
      70   <NA>    NA      NA  NA  <NA>
      71   <NA>    NA      NA  NA  <NA>
      72   <NA>    NA      NA  NA  <NA>
      73   <NA>    NA      NA   1     ?
      74   <NA>    NA      NA  NA  <NA>
      75   <NA>    NA      NA  NA  <NA>
      76   <NA>    NA      NA  NA  <NA>
      77   <NA>    NA      NA  NA  <NA>
      78   <NA>    NA      NA  NA  <NA>
      79   <NA>    NA      NA  NA  <NA>
      80   <NA>    NA      NA  NA  <NA>
      81   <NA>    NA      NA  NA  <NA>
      82   <NA>    NA      NA  NA  <NA>
                                                        tips lwd adjx adjy   lty
      1   <span style='font-size:14px'><b>1_1</b></span><br>   1   NA   NA  <NA>
      2   <span style='font-size:14px'><b>1_3</b></span><br>   1   NA   NA  <NA>
      3   <span style='font-size:14px'><b>1_7</b></span><br>   1   NA   NA  <NA>
      4  <span style='font-size:14px'><b>1_10</b></span><br>   1   NA   NA  <NA>
      5   <span style='font-size:14px'><b>1_2</b></span><br>   1   NA   NA  <NA>
      6   <span style='font-size:14px'><b>1_5</b></span><br>   1   NA   NA  <NA>
      7   <span style='font-size:14px'><b>1_8</b></span><br>   1   NA   NA  <NA>
      8   <span style='font-size:14px'><b>1_6</b></span><br>   1   NA   NA  <NA>
      9   <span style='font-size:14px'><b>1_9</b></span><br>   1   NA   NA  <NA>
      10  <span style='font-size:14px'><b>1_4</b></span><br>   1   NA   NA  <NA>
      11  <span style='font-size:14px'><b>1_1</b></span><br>  NA  0.5  0.5  <NA>
      12  <span style='font-size:14px'><b>1_3</b></span><br>  NA  0.5  0.5  <NA>
      13  <span style='font-size:14px'><b>1_7</b></span><br>  NA  0.5  0.5  <NA>
      14 <span style='font-size:14px'><b>1_10</b></span><br>  NA  0.5  0.5  <NA>
      15  <span style='font-size:14px'><b>1_2</b></span><br>  NA  0.5  0.5  <NA>
      16  <span style='font-size:14px'><b>1_5</b></span><br>  NA  0.5  0.5  <NA>
      17  <span style='font-size:14px'><b>1_8</b></span><br>  NA  0.5  0.5  <NA>
      18  <span style='font-size:14px'><b>1_6</b></span><br>  NA  0.5  0.5  <NA>
      19  <span style='font-size:14px'><b>1_9</b></span><br>  NA  0.5  0.5  <NA>
      20  <span style='font-size:14px'><b>1_4</b></span><br>  NA  0.5  0.5  <NA>
      21  <span style='font-size:14px'><b>1_1</b></span><br>   1   NA   NA  <NA>
      22  <span style='font-size:14px'><b>1_3</b></span><br>   1   NA   NA  <NA>
      23  <span style='font-size:14px'><b>1_7</b></span><br>   1   NA   NA  <NA>
      24 <span style='font-size:14px'><b>1_10</b></span><br>   1   NA   NA  <NA>
      25  <span style='font-size:14px'><b>1_2</b></span><br>   1   NA   NA  <NA>
      26  <span style='font-size:14px'><b>1_5</b></span><br>   1   NA   NA  <NA>
      27  <span style='font-size:14px'><b>1_8</b></span><br>   1   NA   NA  <NA>
      28  <span style='font-size:14px'><b>1_6</b></span><br>   1   NA   NA  <NA>
      29  <span style='font-size:14px'><b>1_9</b></span><br>   1   NA   NA  <NA>
      30  <span style='font-size:14px'><b>1_4</b></span><br>   1   NA   NA  <NA>
      31  <span style='font-size:14px'><b>1_1</b></span><br>  NA  0.5  0.5  <NA>
      32  <span style='font-size:14px'><b>1_3</b></span><br>  NA  0.5  0.5  <NA>
      33  <span style='font-size:14px'><b>1_7</b></span><br>  NA  0.5  0.5  <NA>
      34 <span style='font-size:14px'><b>1_10</b></span><br>  NA  0.5  0.5  <NA>
      35  <span style='font-size:14px'><b>1_2</b></span><br>  NA  0.5  0.5  <NA>
      36  <span style='font-size:14px'><b>1_5</b></span><br>  NA  0.5  0.5  <NA>
      37  <span style='font-size:14px'><b>1_8</b></span><br>  NA  0.5  0.5  <NA>
      38  <span style='font-size:14px'><b>1_6</b></span><br>  NA  0.5  0.5  <NA>
      39  <span style='font-size:14px'><b>1_9</b></span><br>  NA  0.5  0.5  <NA>
      40  <span style='font-size:14px'><b>1_4</b></span><br>  NA  0.5  0.5  <NA>
      41                                                <NA>   1   NA   NA solid
      42                                                <NA>   1   NA   NA solid
      43                                                <NA>   1   NA   NA solid
      44                                                <NA>   1   NA   NA solid
      45  <span style='font-size:14px'><b>1_1</b></span><br>  NA  0.5  1.0  <NA>
      46  <span style='font-size:14px'><b>1_3</b></span><br>  NA  0.5  1.0  <NA>
      47  <span style='font-size:14px'><b>1_7</b></span><br>  NA  0.5  1.0  <NA>
      48 <span style='font-size:14px'><b>1_10</b></span><br>  NA  0.5  1.0  <NA>
      49  <span style='font-size:14px'><b>1_2</b></span><br>  NA  0.5  1.0  <NA>
      50  <span style='font-size:14px'><b>1_5</b></span><br>  NA  0.5  1.0  <NA>
      51  <span style='font-size:14px'><b>1_8</b></span><br>  NA  0.5  1.0  <NA>
      52  <span style='font-size:14px'><b>1_6</b></span><br>  NA  0.5  1.0  <NA>
      53  <span style='font-size:14px'><b>1_9</b></span><br>  NA  0.5  1.0  <NA>
      54  <span style='font-size:14px'><b>1_4</b></span><br>  NA  0.5  1.0  <NA>
      55                                                <NA>   1   NA   NA solid
      56                                                <NA>   1   NA   NA solid
      57                                                <NA>   1   NA   NA solid
      58                                                <NA>   1   NA   NA solid
      59                                                <NA>   1   NA   NA solid
      60                                                <NA>   1   NA   NA solid
      61                                                <NA>   1   NA   NA solid
      62                                                <NA>   1   NA   NA solid
      63                                                <NA>   1   NA   NA solid
      64                                                <NA>   1   NA   NA solid
      65                                                <NA>   1   NA   NA solid
      66                                                <NA>   1   NA   NA solid
      67                                                <NA>   1   NA   NA solid
      68                                                <NA>   1   NA   NA solid
      69                                                <NA>   1   NA   NA solid
      70                                                <NA>   1   NA   NA solid
      71                                                <NA>   1   NA   NA solid
      72                                                <NA>   1   NA   NA solid
      73                                                <NA>  NA  0.5  0.5  <NA>
      74                                                <NA>   1   NA   NA solid
      75                                                <NA>   1   NA   NA solid
      76                                                <NA>   1   NA   NA solid
      77                                                <NA>   1   NA   NA solid
      78                                                <NA>   1   NA   NA solid
      79                                                <NA>   1   NA   NA solid
      80                                                <NA>   1   NA   NA solid
      81                                                <NA>   1   NA   NA solid
      82                                                <NA>   1   NA   NA solid
      
      $par_usr
      $par_usr$usr
      [1] -0.06355  3.06355  4.24794  1.00000
      
      $par_usr$old_par
      $par_usr$old_par$xpd
      [1] TRUE
      
      
      $par_usr$boxw
      [1] 0.1271
      
      $par_usr$boxh
      [1] 0.1319
      
      $par_usr$labh
      [1] 0.05802
      
      $par_usr$legh
      [1] 0.1978
      
      
      $ind_not_plot
      character(0)
      

# Tooltip works

    Code
      html_plot

