% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_fct.R
\name{draw_segment}
\alias{draw_segment}
\title{Draw segments}
\usage{
draw_segment(
  x0,
  y0,
  x1,
  y1,
  ggplot_gen = FALSE,
  col = par("fg"),
  lwd = par("lwd"),
  lty = par("lty")
)
}
\arguments{
\item{x0}{x coordinate of the first point}

\item{y0}{y coordinate of the first point}

\item{x1}{x coordinate of the second point}

\item{y1}{y coordinate of the second point}

\item{ggplot_gen}{If TRUE add the segments to the ggplot object}

\item{col}{Line color}

\item{lwd}{Line width}

\item{lty}{Line type}
}
\value{
Plot the segments to the current device
or add it to a ggplot object
}
\description{
Draw segments
}
\keyword{Pedigree-plot}
\keyword{internal}
