% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_fct.R
\name{draw_polygon}
\alias{draw_polygon}
\title{Draw a polygon}
\usage{
draw_polygon(
  x,
  y,
  ggplot_gen = FALSE,
  fill = "grey",
  border = "black",
  density = NULL,
  angle = 45,
  lwd = par("lwd"),
  tips = NULL
)
}
\arguments{
\item{x}{x coordinates}

\item{y}{y coordinates}

\item{ggplot_gen}{If TRUE add the segments to the ggplot object}

\item{fill}{Fill color}

\item{border}{Border color}

\item{density}{Density of shading}

\item{angle}{Angle of shading}

\item{lwd}{Line width}

\item{tips}{Text to be displayed when hovering over the polygon}
}
\value{
Plot the polygon  to the current device
or add it to a ggplot object
}
\description{
Draw a polygon
}
\keyword{Pedigree-plot}
\keyword{internal}
