% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/app_utils.R
\name{make_reactive}
\alias{make_reactive}
\title{Make a reactive object}
\usage{
make_reactive(x)
}
\arguments{
\item{x}{The input to check and convert.}
}
\value{
A reactive object.
}
\description{
This function checks if the input is a reactive object. If it is, it returns
it as is. If not, it converts the input into a reactive object.
}
\examples{
Pedixplorer:::make_reactive(1)
}
\keyword{internal}
