% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_fct.R
\name{draw_text}
\alias{draw_text}
\title{Draw texts}
\usage{
draw_text(
  x,
  y,
  label,
  ggplot_gen = FALSE,
  cex = par("cex"),
  col = par("col"),
  adjx = 0.5,
  adjy = 0.5,
  tips = NULL
)
}
\arguments{
\item{x}{x coordinates}

\item{y}{y coordinates}

\item{label}{Text to be displayed}

\item{ggplot_gen}{If TRUE add the segments to the ggplot object}

\item{cex}{Character expansion of the text}

\item{col}{Text color}

\item{adjx}{x adjustment}

\item{adjy}{y adjustment}

\item{tips}{Text to be displayed when hovering over the text}
}
\value{
Plot the text to the current device
or add it to a ggplot object
}
\description{
Draw texts
}
\keyword{Pedigree-plot}
\keyword{internal}
